import EmberArray, { A } from "@ember/array";
import MutableArray from "@ember/array/mutable";
import EmberObject from "@ember/object";
import { assertType } from "./lib/assert";

type Person = EmberObject & { name: string; isHappy: boolean };
const Person = EmberObject.extend({
    name: "",
    isHappy: false,
});

const people = A([Person.create({ name: "Yehuda", isHappy: true }), Person.create({ name: "Majd", isHappy: false })]);

assertType<number>(people.get("length"));
assertType<Person | undefined>(people.get("lastObject"));
assertType<Person | undefined>(people.get("firstObject"));
assertType<boolean>(people.isAny("isHappy"));
assertType<boolean>(people.isAny("isHappy", false));
// @ts-expect-error
assertType<boolean>(people.isAny("isHappy", "false"));

assertType<Person | undefined>(people.objectAt(0));
assertType<EmberArray<Person | undefined>>(people.objectsAt([1, 2, 3]));

const persons1: Person[] = people.filterBy("isHappy");
const persons2: MutableArray<Person> = people.filterBy("isHappy");
const persons3: Person[] = people.rejectBy("isHappy");
const persons4: MutableArray<Person> = people.rejectBy("isHappy");
const persons5: Person[] = people.filter(person => person.get("name") === "Yehuda");
const persons6: MutableArray<Person> = people.filter(person => person.get("name") === "Yehuda");

assertType<typeof people>(people.get("[]"));
assertType<Person | undefined>(people.get("[]").get("firstObject"));

assertType<boolean[]>(people.mapBy("isHappy"));
assertType<unknown[]>(people.mapBy("name.length"));

const last = people.get("lastObject"); // $ExpectType ({ name: string; isHappy: boolean; } & EmberObject & { name: string; isHappy: boolean; }) | undefined
if (last) {
    assertType<string>(last.get("name"));
}

const first = people.get("lastObject");
if (first) {
    assertType<boolean>(first.get("isHappy"));
}

const letters: EmberArray<string> = A(["a", "b", "c"]);
const codes: number[] = letters.map((item, index, enumerable) => {
    assertType<string>(item);
    assertType<number>(index);
    return item.charCodeAt(0);
});

const value = "1,2,3";
const filters = A(value.split(","));
filters.push("4");
filters.sort();

const multiSortArr = A([
    { k: "a", v: "z" },
    { k: "a", v: "y" },
    { k: "b", v: "c" },
]);
multiSortArr.sortBy("k", "v");
